# Generated by Django 4.0.4 on 2022-05-12 23:39

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_mnsi", "0002_historicalmnsi_mnsi"),
    ]

    operations = [
        migrations.AlterField(
            model_name="mnsi",
            name="abnormal_obs_left_foot",
            field=models.ManyToManyField(
                blank=True,
                related_name="+",
                to="edc_mnsi.abnormalfootappearanceobservations",
                verbose_name="If NO, check all that apply to LEFT foot?",
            ),
        ),
        migrations.AlterField(
            model_name="mnsi",
            name="abnormal_obs_right_foot",
            field=models.ManyToManyField(
                blank=True,
                related_name="+",
                to="edc_mnsi.abnormalfootappearanceobservations",
                verbose_name="If NO, check all that apply to RIGHT foot?",
            ),
        ),
    ]
